******************************************************************************
*       UniROM.S - Copyright (c) 1996 by TechTools, all rights reserved.
*       Based on the template DEVICE.S
*
*       DEVICE.S - template for device-specific definitions & options
*       Copyright (c) 1990-1991, Software Development Systems, Inc.
*       All Rights Reserved.
*
*       This file defines the following routines, each of which is prefixed
*       by the device name (see "NAME" below):
*
*          Initialization:
*               init    initializes the entire device (all ports)
*          Enabling:
*               eio     enable a port for both input & output
*          Output:
*               put     poll transmitter & emit character
*          Input:
*               poll    poll for input character
*               get     get input character (assume character present)
*
******************************************************************************
		.OPTION flags=.

#define NAME UNIROM

******************************************************************************
*       INCLUDES
******************************************************************************
		SECTION usr_code
		.option list="off"
		include com.h
		include device.h
		.option list="on"

******************************************************************************
*       DEVICE-SPECIFIC DEFINITIONS
******************************************************************************
       
		; URCOM registers
		; URCOM is the virtual UART within UniROM

	STATUS:     equ     OFF13       ; STATUS REGISTER
	RX_BUFF:    equ     OFF12       ; RX data BUFFER
	TX_FULL:    equ     OFF8        ; Start of SET TX FLAG region

******************************************************************************
*       INITIALIZATION FUNCTIONS
******************************************************************************

	; **************************************************
	;  Device initialization routine.
	;
	;  INITIALIZE THE ENTIRE DEVICE, regardless of
	;  which port(s) is(are) being used. 
	;
	;  INPUT PARAMETERS:
	;       Device base address in A0
	;  RETURN VALUE: none
	; **************************************************
		SECTION usr_code

\{NAME}init:    tst.b   RX_BUFF(a0)     ; Clear any pending RX
		move.b  STATUS(a0),d0   ; Get status
		and.b   #3,d0           ; Isolate the status flags
		bne.s  \{NAME}init      ; If URCOM not initialized, keep checking
		rts

******************************************************************************
*       OUTPUT FUNCTIONS
******************************************************************************

	; **************************************************
	;  Enable port for output only.
	;     No initialization is required to enable 
	;     the output port - simply return.
	; **************************************************
		SECTION usr_code
\{NAME}eo:     rts

	; **************************************************
	;  Wait until "clear to send".
	;
	;  INPUT PARAMETERS:
	;       Device base address in A0
	;       Port base address in A1
	;  RETURN VALUE: none; waits for "clear"
	; **************************************************
		SECTION usr_code

	; HARDWARE Handshaking is automatic with URCOM. The 
	; TX buffer stays full until the HOST asserts a
	; "clear to send" and accepts the last character sent.
	; Therefore, simply return.
\{NAME}ctsw:    rts


	; **************************************************
	;  Put out a character. MUST CHECK TO ENSURE THAT
	;  THE TRANSMITTER IS READY (unlike "get", which
	;  does not check to ensure the receiver is ready).
	;
	;  INPUT PARAMETERS:
	;       Device base address in A0
	;       Port base address in A1
	;       Character to be transmitted in bottom of D0
	;  RETURN VALUE: none
	; **************************************************
		SECTION usr_code
\{NAME}put:             
			; wait for TX buffer to empty
		move.b  STATUS(a0),d1   ; Get status
		and.b   #1,d1           ; Isolate the TXFULL flag
		bne.s  \{NAME}put       ; If TXFULL, keep checking
			
			; TX buffer empty, now SEND the data
		move    d0,d1           ; make a working copy of data
		and.l   #7,d1           ; isolate lower 3 bits
		mulu.w  #OFF1,d1        ; scale address index
		tst.b   (a0,d1)         ; encode b0-b2
		ror.b   #3,d0           ; get next 3 bits           
		move    d0,d1           ; make a working copy
		and.l   #7,d1           ; isolate b3-b5
		mulu.w  #OFF1,d1        ; scale address index
		tst.b   (a0,d1)         ; encode b3-b5
		ror.b   #3,d0           ; get last 2 bits
		and.l   #3,d0           ; isolate b6 & b7
		mulu.w  #OFF1,d0        ; scale address index
		tst.b   TX_FULL(a0,d0)  ; encode b6-b7 and set TXFULL
		rts

******************************************************************************
*       INPUT FUNCTIONS
******************************************************************************

	; **************************************************
	;  Enable device for input only.
	;       No initialization is required to enable 
	;       the input port - simply return.
	; **************************************************
		SECTION usr_code
\{NAME}ei:      rts


	; **************************************************
	;  Poll once to see if a character is present.
	;
	;  INPUT PARAMETERS:
	;       Device base address in A0
	;       Port base address in A1
	;  RETURN VALUE:
	;       nonzero in D0 if char is present
	;       zero in D0 if char is not present
	; **************************************************
		SECTION usr_code
\{NAME}poll:    move.b STATUS(a0),d0   ; Get status
		and.l   #2,d0           ; Isolate the RXFULL flag
		rts                     ; 0->empty, 2->data present

	; **************************************************
	;  Get a character from the input port. ASSUME THAT
	;  WE HAVE ALREADY CHECKED FOR A VALID CHARACTER
	;  (i.e. DO NOT POLL).
	;
	;  INPUT PARAMETERS:
	;       Device base address in A0
	;       Port base address in A1
	;  RETURN VALUE:
	;       character in bottom byte of D0
	; **************************************************
		SECTION usr_code
\{NAME}get:     move.b  RX_BUFF(a0),d0    ; get & return character 
		rts


******************************************************************************
*       SPECIAL INPUT FUNCTIONS
******************************************************************************

	; **************************************************
	;  Clear error status (from a "read").
	;
	;  INPUT PARAMETERS:
	;       Device base address in A0
	;       Port base address in A1
	;  RETURN VALUE: none
	;
	;  No specific action is required to clear errors.
	;  Simply RETURN 
	;
	; **************************************************
		SECTION usr_code
\{NAME}cerr:    rts


	; **************************************************
	;  Clear interrupt status (from a "read").
	;
	;  Receive Interrupts are automatically cleared by the
	;  read operation itself. Simply RETURN 
	;
	; **************************************************
		SECTION usr_code
\{NAME}cint:    rts

******************************************************************************
*       COMBINED I/O INITIALIZATION
******************************************************************************

	; **************************************************
	;  Enable the same device for both input and output.
	;  UniROM/URCOM does not require any  specific initialization 
	;  to enable combined I/O.  Simply RETURN.
	;
	; **************************************************
		SECTION usr_code
\{NAME}eio:   rts

******************************************************************************
*       DEBUG
*
*       The routine below dumps all the internal registers of the
*       appropriate port. It should probably call on the "Emit"
*       routine provided in "debug/debug.s", which in turn calls
*       upon "put" (above) to put the values out. The "Emit"
*       routine converts the value of the lowest byte of D0 and
*       prints it out as two ASCII characters. For example, if
*       the lowest byte of D0 contains the value "7A", Emit will
*       call "put" with an ASCII "7" followed by an ASCII "A".
*
*       Note that if you are using the same device for input and
*       output, the output portion must be working before this type
*       of debugging can be used.
*
*       These routines are never used by a normal application; they
*       exist so that you can call on them when debugging code that
*       you are writing for a new device.
******************************************************************************
#ifdef DEBUG
	; **************************************************
	;  Emit all internal registers of the port
	;
	;  INPUT PARAMETERS:
	;       Device base address in A0
	;       Port base address in A1
	;  RETURN VALUE: none
	; **************************************************
		SECTION usr_code
\{NAME}emit:    ; not implemented, simply return
		rts
#endif


******************************************************************************
*    Option line: leave this here: it allows this file to be compiled
*    and linked without warnings to the monitor, regardless of what
*    processor you are using.
******************************************************************************
		opt     p=68000
